<?php
//
// Created on: <16-Apr-2006 18:52:01 dl>
//
// Copyright (C) 1999-2006 eZ systems as. All rights reserved.
//
// This source file is part of the eZ publish (tm) Open Source Content
// Management System.
//
// This file may be distributed and/or modified under the terms of the
// "GNU General Public License" version 2 as published by the Free
// Software Foundation and appearing in the file LICENSE included in
// the packaging of this file.
//
// Licencees holding a valid "eZ publish professional licence" version 2
// may use this file in accordance with the "eZ publish professional licence"
// version 2 Agreement provided with the Software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
// THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
// The "eZ publish professional licence" version 2 is available at
// http://ez.no/ez_publish/licences/professional/ and in the file
// PROFESSIONAL_LICENCE included in the packaging of this file.
// For pricing of this licence please contact us via e-mail to licence@ez.no.
// Further contact information is available at http://ez.no/company/contact/.
//
// The "GNU General Public License" (GPL) is available at
// http://www.gnu.org/copyleft/gpl.html.
//
// Contact licence@ez.no if any conditions of this licencing isn't clear to
// you.
//

function eZSitePreInstall()
{
    eZSetupMultipriceProducts();
    eZSetupDynamicVATProducts();
}

function eZSetupMultipriceProducts()
{
    // create currencies for 'multiprice_products' package
    include_once( 'kernel/shop/classes/ezcurrencydata.php' );

    $neededCurrencies = array( 'NOK' => array( 'symbol' => 'kr',
                                               'locale' => 'nor-NO',
                                               'auto_rate_value' => '7.8562',
                                               'custom_rate_value' => '0.0000',
                                               'rate_factor' => '1.0000' ),
                               'USD' => array( 'symbol' => '$',
                                               'locale' => 'eng-US',
                                               'auto_rate_value' => '1.2094',
                                               'custom_rate_value' => '0.0000',
                                               'rate_factor' => '1.0000' ) );

    $currencyList = eZCurrencyData::fetchList();

    $db =& eZDB::instance();
    $db->begin();

    foreach ( $neededCurrencies as $currencyCode => $currencyData )
    {
        if ( !isset( $currencyList[$currencyCode] ) )
        {
            $currency = eZCurrencyData::create( $currencyCode,
                                                $currencyData['symbol'],
                                                $currencyData['locale'],
                                                $currencyData['auto_rate_value'],
                                                $currencyData['custom_rate_value'],
                                                $currencyData['rate_factor'] );

            if ( is_object( $currency ) )
                $currency->store();
            else
                eZDebug::writeError( "Unable to create currency '$currencyCode' for 'multiprice_products' packages", 'eZSetupMultipriceProducts' );
        }
    }

    $db->commit();
}

function eZSetupDynamicVATProducts()
{
    // Create a few VAT types.
    require_once( 'kernel/classes/ezvattype.php' );

    $vatTypeDataArray = array(
        'Norway general'                => array( 'percentage' => 25 ),
        'Norway reduced'                => array( 'percentage' => 11 ),
        'Norway low, Germany reduced'   => array( 'percentage' => 7 ),
        'Germany general'               => array( 'percentage' => 16 ) );

    foreach ( $vatTypeDataArray as $vatName => $vatData )
    {
        $vat = eZVatType::create();
        $vat->setAttribute( 'name', $vatName );
        $vat->setAttribute( 'percentage', $vatData['percentage'] );
        $vat->store();
        $vatTypeDataArray[$vatName]['id'] = $vat->attribute( 'id' );
    }

    // Create few product categories.
    require_once( 'kernel/classes/ezproductcategory.php' );

    $productCategoryDataArray = array(
        'Food'               => array(),
        'Personal transport' => array() );

    foreach ( $productCategoryDataArray as $catName => $catData )
    {
        $cat = eZProductCategory::create();
        $cat->setAttribute( 'name', $catName );
        $cat->store();
        $productCategoryDataArray[$catName]['id'] = $cat->attribute( 'id' );
    }

    // Create a few VAT charging rules.
    require_once( 'kernel/classes/ezvatrule.php' );

    $vatRuleDataArray = array(
        array( 'country' => 'Germany', 'categories' => array( 'Food' ),               'vat_type_name' => 'Norway low, Germany reduced' ),
        array( 'country' => 'Germany', 'categories' => array(),                       'vat_type_name' => 'Germany general' ),
        array( 'country' => 'Norway',  'categories' => array( 'Personal transport' ), 'vat_type_name' => 'Norway low, Germany reduced' ),
        array( 'country' => 'Norway',  'categories' => array( 'Food' ),               'vat_type_name' => 'Norway reduced' ),
        array( 'country' => 'Norway',  'categories' => array(),                       'vat_type_name' => 'Norway general' ),
        array( 'country' => '*',       'categories' => array(),                       'vat_type_name' => 'Std' ) );

    foreach ( $vatRuleDataArray as $ruleData )
    {
        $rule = eZVatRule::create();

        $vatTypeName = $ruleData['vat_type_name'];
        $vatTypeID = ( $vatTypeName == 'Std' ? 1 : $vatTypeDataArray[$vatTypeName]['id'] );

        $rule->setAttribute( 'country', $ruleData['country'] );
        $rule->setAttribute( 'vat_type', $vatTypeID );
        $categories = array();
        foreach ( $ruleData['categories'] as $catName )
            $categories[] = $productCategoryDataArray[$catName];
        $rule->setAttribute( 'product_categories', $categories );
        $rule->store();
    }

    // Add country attribute to the User class.
    eZAddClassAttribute( array( 'class_remote_id'  => '40faa822edc579b02c25f6bb7beec3ad',
                                'data_type_string' => 'ezcountry',
                                'attr_identifier'  => 'country',
                                'attr_name'        => 'Country' ) );

    /*
    // Add category attribute to the Product class
    eZAddClassAttribute( array( 'class_remote_id'  => '0115ac3b3e93096c29f2f4ff65b08856',
                                'data_type_string' => 'ezproductcategory',
                                'attr_identifier'  => 'category',
                                'attr_name'        => 'Category' ) );
    */
}

function eZAddClassAttribute( $parameters )
{
    require_once( 'kernel/classes/ezcontentclass.php' );

    $class     = eZContentClass::fetchByRemoteID( $parameters['class_remote_id'] );
    $className = $class->attribute( 'name' );
    $classID   = $class->attribute( 'id' );
    $attrIdentifier = $parameters['attr_identifier'];
    $attrName       = $parameters['attr_name'];
    $attrDatatype   = $parameters['data_type_string'];

    // Check if the class already contains an attribute of the same datatype and identifier.
    $filteredAttrs = eZContentClassAttribute::fetchFilteredList( array( 'contentclass_id' => $classID,
                                                                        'version' => EZ_CLASS_VERSION_STATUS_DEFINED,
                                                                        'data_type_string' => $attrDatatype,
                                                                        'identifier' => $attrIdentifier ) );
    if ( $filteredAttrs )
    {
        eZDebug::writeWarning( "The <$className> class already contains <$attrName> attribute." );
        return;
    }

    // Add Country attribute to the class.
    require_once( 'kernel/classes/ezcontentclassattribute.php' );
    eZDebug::writeDebug( "Adding attribute <$attrName> of type <$attrDatatype> to class <$className>" );
    $newAttr = eZContentClassAttribute::create( $classID, $attrDatatype );
    $newAttr->setAttribute( 'name', $attrName );
    $newAttr->setAttribute( 'identifier', $attrIdentifier );
    $newAttr->setAttribute( 'version', EZ_CLASS_VERSION_STATUS_DEFINED );
    $newAttrPlacement = eZPersistentObject::newObjectOrder( eZContentClassAttribute::definition(),
                                                                'placement',
                                                                array( 'version' => EZ_CLASS_VERSION_STATUS_DEFINED,
                                                                       'contentclass_id' => $classID ) );
    $newAttr->setAttribute( 'placement', $newAttrPlacement );
    $newAttrDataType = $newAttr->dataType();
    $newAttrDataType->initializeClassAttribute( $newAttr );
    $newAttr->store();
    $newAttrID = $newAttr->attribute( 'id' );

    // Add the attribute to all existing objects of the class.
    require_once( 'kernel/classes/ezcontentobject.php' );
    require_once( 'kernel/classes/ezcontentobjectattribute.php' );
    $objects = eZContentObject::fetchSameClassList( $classID );
    foreach ( $objects as $object )
    {
        $contentobjectID = $object->attribute( 'id' );
        $objectVersions =& $object->versions();
        eZDebug::writeDebug( sprintf( "Adding attribute <$attrName> to object #%d <%s>",
                                      $contentobjectID, $object->attribute( 'name' ) ) );
        foreach ( $objectVersions as $objectVersion )
        {
            $translations = $objectVersion->translations( false );
            $version = $objectVersion->attribute( 'version' );
            foreach ( $translations as $translation )
            {
                $objectAttribute = eZContentObjectAttribute::create( $newAttrID, $contentobjectID, $version );
                $objectAttribute->setAttribute( 'language_code', $translation );
                $objectAttribute->initialize();
                $objectAttribute->store();
                $objectAttribute->postInitialize();
            }
        }
    }
}
?>
